@@ -1254,7 +1254,8 @@ while(defined(my $line=<>)) {
 print "\n$current_segment\tENDS\n"	if ($current_segment && $masm);
 print "END\n"				if ($masm);
 print "#endif\n"			if ($gas);
-
+# See https://www.airs.com/blog/archives/518.
+print ".section\t.note.GNU-stack,\"\",\@progbits\n" if ($elf);
 
 close STDOUT or die "error closing STDOUT";
 
